import sys, os, time, math
from scitbx.array_family import flex
import iotbx.pdb
import scitbx.math, random
import mmtbx.real_space
from cctbx import maptbx
import scitbx.math
from cctbx import miller
from mmtbx.maps import kick, fem
from libtbx.utils import null_out
from mmtbx import map_tools
from libtbx.test_utils import approx_equal, not_approx_equal
import mmtbx.masks
    

def get_map(mc, cg, volume_scale=False):
  fft_map = miller.fft_map(
      crystal_gridding     = cg,
      fourier_coefficients = mc)
  if(volume_scale):
    fft_map.apply_volume_scaling()
  else:
    fft_map.apply_sigma_scaling()
  return fft_map.real_map_unpadded()

def ccp4_map(cg, file_name, mc=None, map_data=None):
  assert [mc, map_data].count(None)==1
  if(map_data is None):
    map_data = get_map(mc=mc, cg=cg)
  from iotbx import ccp4_map
  ccp4_map.write_ccp4_map(
      file_name=file_name,
      unit_cell=cg.unit_cell(),
      space_group=cg.space_group(),
      #gridding_first=(0,0,0),# This causes a bug (map gets shifted)
      #gridding_last=n_real,  # This causes a bug (map gets shifted)
      map_data=map_data,
      labels=flex.std_string([""]))
  
def run(args):
  inputs = mmtbx.utils.process_command_line_args(args = args)
  df = mmtbx.utils.determine_data_and_flags(
    reflection_file_server  = inputs.get_reflection_file_server(),
    log = null_out())
  pdb_inp = iotbx.pdb.input(file_name=inputs.pdb_file_names[0])
  xrs = pdb_inp.xray_structure_simple()
  #########
  #from cctbx import adptbx
  #b_iso_min = flex.min(xrs.extract_u_iso_or_u_equiv()*adptbx.u_as_b(1))
  #print "Max B subtracted from atoms and used to sharpen map:", b_iso_min
  #xrs.shift_us(b_shift=-b_iso_min)
  #b_iso_min = flex.min(xrs.extract_u_iso_or_u_equiv()*adptbx.u_as_b(1))
  #assert approx_equal(b_iso_min, 0, 1.e-3)
  #
  uc = xrs.unit_cell()
  #########
  f_obs = df.f_obs.average_bijvoet_mates()
  f_obs, r_free_flags = f_obs.common_sets(df.r_free_flags)
  fmodel = mmtbx.f_model.manager(
    f_obs          = f_obs,
    r_free_flags   = r_free_flags,
    xray_structure = xrs)
  fmodel.update_all_scales(update_f_part1=False) # XXX
  print fmodel.r_work(), fmodel.r_free()
  fmodel.show(show_header=False, show_approx=False)
  #
  crystal_gridding = fmodel.f_obs().crystal_gridding(
    d_min             = fmodel.f_obs().d_min(),
    symmetry_flags    = maptbx.use_space_group_symmetry,
    resolution_factor = 1./4)
  # orig
  mc = map_tools.electron_density_map(
    fmodel=fmodel).map_coefficients(
      map_type         = "2mFo-DFc",
      isotropize       = True,
      fill_missing     = False)
  crom = mmtbx.maps.composite_omit_map.run(
    crystal_gridding = crystal_gridding,
    box_size_as_fraction = 0.001,
    fmodel           = fmodel)
  
  p1_map = crom.as_p1_map()
  p1_map_nf = crom.as_p1_map_noise_filtered()
  ccp4_map(cg=crystal_gridding, file_name="omit.ccp4", map_data=p1_map)
  ccp4_map(cg=crystal_gridding, file_name="omit_noise.ccp4", map_data=p1_map_nf)
  
  mc_omit = crom.map_coefficients

  mtz_dataset = mc.as_mtz_dataset(column_root_label="2mFo-DFc")
  mtz_dataset.add_miller_array(
    miller_array      = mc_omit,
    column_root_label = "OMIT")
  mtz_dataset.add_miller_array(
    miller_array      = crom.noise_filtered_map_coefficients(),
    column_root_label = "OMIT_NF")
  mtz_object = mtz_dataset.mtz_object()
  mtz_object.write(file_name = "omit.mtz")

  

if (__name__ == "__main__"):
  t0 = time.time()
  run(sys.argv[1:])
  print "Time:", time.time()-t0

